/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_BINARY_FILE_H
#define O2SCL_BINARY_FILE_H

#include <iostream>
#include <o2scl/misc.h>
#include <o2scl/err_hnd.h>
#include <o2scl/file_format.h>
#include <o2scl/lib_settings.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Binary output file


   */
  class binary_out_file : public out_file_format {
  public:

    /** \brief Create a binary output file with name \c file_name  
     */
    binary_out_file(std::string file_name);

    virtual ~binary_out_file();

    /** \brief Output a bool variable
     */
    virtual int bool_out(bool dat, std::string name="");

    /** \brief Output a char variable
     */
    virtual int char_out(char dat, std::string name="");

    /** \brief Output a double variable
     */
    virtual int double_out(double dat, std::string name="");

    /** \brief Output a float variable
     */
    virtual int float_out(float dat, std::string name="");

    /** \brief Output an int variable
     */
    virtual int int_out(int dat, std::string name="");

    /** \brief Output an long variable
     */
    virtual int long_out(unsigned long int dat, std::string name="");

    /** \brief Output a string
     */
    virtual int string_out(std::string dat, std::string name="");

    /** \brief Output a word
     */
    virtual int word_out(std::string dat, std::string name="");

    /// Start an object
    virtual int start_object(std::string type, std::string name="");
    
    /** \brief End object output (does nothing for a binary file)
     */
    virtual int end_object();

    /** \brief End a line of output (does nothing for a binary file)
     */
    virtual int end_line();
  
    /** \brief Output initialization */
    virtual int init_file();

    /// Finish the file
    virtual int clean_up() {
      return 0;
    }

#ifndef DOXYGEN_INTERNAL

  protected:

    friend class collection;
  
    /// True if the file is to be compressed
    bool compressed;

    /// True if the compression is to be performed by gzip
    bool gzip;

    /// The output stream
    std::ofstream outs;

    /// The filename specified by the user
    std::string user_filename;

    /// The temporary filename
    std::string temp_filename;

    /// \name The output format
    //@{
    int fill, precision;
    //std::_Ios_Fmtflags flags;
    //@}

#endif
			   
  };

  /** 
      \brief Binary input file


   */
  class binary_in_file : public in_file_format {
  public:

    /** \brief Read an input file with name \c file_name  */
    binary_in_file(std::string file_name);

    virtual ~binary_in_file();
  
    virtual int bool_in(bool &dat, std::string name="");
    virtual int char_in(char &dat, std::string name="");
    virtual int double_in(double &dat, std::string name="");
    virtual int float_in(float &dat, std::string name="");
    virtual int int_in(int &dat, std::string name="");
    virtual int long_in(unsigned long int &dat, std::string name="");
    virtual int string_in(std::string &dat, std::string name="");
    virtual int word_in(std::string &dat, std::string name="");
    
    /// Read the initialization
    virtual int init_file();

    /// Clean up the file
    virtual int clean_up() {
      return 0;
    }
    
    /// Begin reading an object
    virtual int start_object(std::string &type, std::string &name);
    
    virtual int skip_object();

    /// Finish reading an object
    virtual int end_object() {
      return 0;
    }

#ifndef DOXYGEN_INTERNAL

  protected:

    /// The input stream
    std::ifstream ins;

#endif
  };

}
#endif
