/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/cx_arith.h>
#include <o2scl/ovector_tlate.h>
#include <o2scl/ovector_cx_tlate.h>
#include <o2scl/omatrix_tlate.h>
#include <o2scl/omatrix_cx_tlate.h>
#include <o2scl/columnify.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_arith;

int main(void) {
  test_mgr t;
  t.set_output_level(1);
  
  // -----------------------------------------------
  // Test arithmetic operators for gsl_complex
  // -----------------------------------------------

  gsl_complex g1={{1.0,2.0}}, g2={{0.5,-1.0}};
  std::complex<double> c1(1.0,2.0), c2(0.5,-1.0);
  double val=3.0;

  // -----------------------------------------------

  gsl_complex gsum=g1+g2;
  std::complex<double> csum=c1+c2;
  gsl_complex gdif=g1-g2;
  std::complex<double> cdif=c1-c2;
  gsl_complex gpro=g1*g2;
  std::complex<double> cpro=c1*c2;
  gsl_complex gquo=g1/g2;
  std::complex<double> cquo=c1/c2;

  t.test_rel(GSL_REAL(gsum),csum.real(),1.0e-15,"sumr");
  t.test_rel(GSL_IMAG(gsum),csum.imag(),1.0e-15,"sumi");
  t.test_rel(GSL_REAL(gdif),cdif.real(),1.0e-15,"difr");
  t.test_rel(GSL_IMAG(gdif),cdif.imag(),1.0e-15,"difi");
  t.test_rel(GSL_REAL(gpro),cpro.real(),1.0e-15,"pror");
  t.test_rel(GSL_IMAG(gpro),cpro.imag(),1.0e-15,"proi");
  t.test_rel(GSL_REAL(gquo),cquo.real(),1.0e-15,"quor");
  t.test_rel(GSL_IMAG(gquo),cquo.imag(),1.0e-15,"quoi");
  
  // -----------------------------------------------

  gsl_complex hsum=g1+val;
  gsl_complex isum=val+g1;
  std::complex<double> dsum=c1+val;
  std::complex<double> esum=val+c1;
  gsl_complex hdif=g1-val;
  gsl_complex idif=val-g1;
  std::complex<double> ddif=c1-val;
  std::complex<double> edif=val-c1;
  gsl_complex hpro=g1*val;
  gsl_complex ipro=val*g1;
  std::complex<double> dpro=c1*val;
  std::complex<double> epro=val*c1;
  gsl_complex hquo=g1/val;
  std::complex<double> dquo=c1/val;

  t.test_rel(GSL_REAL(hsum),dsum.real(),1.0e-15,"sumr");
  t.test_rel(GSL_IMAG(hsum),dsum.imag(),1.0e-15,"sumi");
  t.test_rel(GSL_REAL(isum),esum.real(),1.0e-15,"sumr");
  t.test_rel(GSL_IMAG(isum),esum.imag(),1.0e-15,"sumi");
  t.test_rel(GSL_REAL(hdif),ddif.real(),1.0e-15,"difr");
  t.test_rel(GSL_IMAG(hdif),ddif.imag(),1.0e-15,"difi");
  t.test_rel(GSL_REAL(idif),edif.real(),1.0e-15,"difr");
  t.test_rel(GSL_IMAG(idif),edif.imag(),1.0e-15,"difi");
  t.test_rel(GSL_REAL(hpro),dpro.real(),1.0e-15,"pror");
  t.test_rel(GSL_IMAG(hpro),dpro.imag(),1.0e-15,"proi");
  t.test_rel(GSL_REAL(ipro),epro.real(),1.0e-15,"pror");
  t.test_rel(GSL_IMAG(ipro),epro.imag(),1.0e-15,"proi");
  t.test_rel(GSL_REAL(hquo),dquo.real(),1.0e-15,"quor");
  t.test_rel(GSL_IMAG(hquo),dquo.imag(),1.0e-15,"quoi");

  // -----------------------------------------------

  gsum+=g1;
  csum+=c1;
  gdif-=g1;
  cdif-=c1;
  gpro*=g1;
  cpro*=c1;
  gquo/=g1;
  cquo/=c1;

  t.test_rel(GSL_REAL(gsum),csum.real(),1.0e-15,"sumr2");
  t.test_rel(GSL_IMAG(gsum),csum.imag(),1.0e-15,"sumi2");
  t.test_rel(GSL_REAL(gdif),cdif.real(),1.0e-15,"difr2");
  t.test_rel(GSL_IMAG(gdif),cdif.imag(),1.0e-15,"difi2");
  t.test_rel(GSL_REAL(gpro),cpro.real(),1.0e-15,"pror2");
  t.test_rel(GSL_IMAG(gpro),cpro.imag(),1.0e-15,"proi2");
  t.test_rel(GSL_REAL(gquo),cquo.real(),1.0e-15,"quor2");
  t.test_rel(GSL_IMAG(gquo),cquo.imag(),1.0e-15,"quoi2");

  // -----------------------------------------------

  double z=2.0;
  gsl_complex gc={{-2.0,0.0}};
  cout << sqrt(z) << " " << sqrt(gc).dat[0] << " " << sqrt(gc).dat[1] << endl;
  gsl_complex gd={{-2.0,1.0}};
  cout << cosh(z) << " " << cosh(gd).dat[0] << " " << cosh(gd).dat[1] << endl;
  std::complex<double> ge(-2.0,1.0);
  cout << cosh(ge).real() << " " << cosh(ge).imag() << endl;
  gsl_complex gf={{-2.0,1.0}};
  cout << acosh(z) << " " << acosh(gf).dat[0] 
       << " " << acosh(gf).dat[1] << endl;
  
  t.report();
  
  return 0;
}

