/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_LIB_SETTINGS_H
#define O2SCL_LIB_SETTINGS_H
#include <iostream>
#include <string>

/** \file lib_settings.h
    \brief File for definitions for lib_settings_class
*/

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** \brief A class to manage testing and record success and failure

  */
  class lib_settings_class {

  public:

    lib_settings_class();

    ~lib_settings_class();

    /** \brief Return the data directory */
    std::string get_data_dir() {
      return data_dir;
    }

    /** \brief Set the data directory */
    int set_data_dir(std::string dir) {
      data_dir=dir;
      return 0;
    }
    
    /** \brief Return the temp file directory */
    std::string get_tmp_dir() {
      return tmp_dir;
    }

    /** \brief Set the temp file directory */
    int set_tmp_dir(std::string dir) {
      tmp_dir=dir;
      return 0;
    }

    /** 
	\brief Return true if range checking was turned on during installation
     */
    bool range_check();

    /// Return the library version
    std::string o2scl_version();

  protected:

#ifndef DOXYGEN_INTERNAL

    /// The present data directory
    std::string data_dir;
    /// The present temp file directory
    std::string tmp_dir;

#endif
  
  };

  /// The global library settings object
  extern lib_settings_class lib_settings;

#ifndef DOXYGENP
}
#endif

#endif
