/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/search_vec.h>

#include <o2scl/test_mgr.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  size_t i;
  search_vec<ovector> ib;

  ovector x(10);

  for(i=0;i<10;i++) {
    x[i]=((double)(i));
  }

  t.test_gen(ib.ordered_lookup(-5.5,10,x)==0,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(15.5,10,x)==9,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(5.0,10,x)==5,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(5.49,10,x)==5,"ordered_lookup (inc)");
  t.test_gen(ib.ordered_lookup(5.51,10,x)==6,"ordered_lookup (inc)");

  t.test_gen(ib.ordered_interval(-5.5,10,x)==0,"ordered_interval (inc)");
  t.test_gen(ib.ordered_interval(15.5,10,x)==9,"ordered_interval (inc)");
  t.test_gen(ib.ordered_interval(5.0,10,x)==5,"ordered_interval (inc)");
  t.test_gen(ib.ordered_interval(5.5,10,x)==5,"ordered_interval (inc)");
  t.test_gen(ib.ordered_interval(5.51,10,x)==5,"ordered_interval (inc)");

  for(i=0;i<10;i++) {
    x[i]=9.0-((double)(i));
  }

  t.test_gen(ib.ordered_lookup(-5.5,10,x)==9,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(15.5,10,x)==0,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(5.0,10,x)==4,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(5.5,10,x)==3,"ordered_lookup (dec)");
  t.test_gen(ib.ordered_lookup(5.51,10,x)==3,"ordered_lookup (dec)");

  t.test_gen(ib.ordered_interval(-5.5,10,x)==9,"ordered_interval (dec)");
  t.test_gen(ib.ordered_interval(15.5,10,x)==0,"ordered_interval (dec)");
  t.test_gen(ib.ordered_interval(5.0,10,x)==4,"ordered_interval (dec)");
  t.test_gen(ib.ordered_interval(5.5,10,x)==3,"ordered_interval (dec)");
  t.test_gen(ib.ordered_interval(5.51,10,x)==3,"ordered_interval (dec)");
  
  for(i=0;i<10;i++) {
    x[i]=((double)(i));
  }
  
  cout << ib.find_inc(-1.0,10,x) << " " 
       << ib.ordered_lookup(-1.0,10,x) << " " 
       << ib.ordered_interval(-1.0,10,x) << " " 
       << ib.bsearch_inc(-1.0,x,0,10) << endl; 
  cout << ib.find_inc(0.0,10,x) << " "
       << ib.ordered_lookup(0.0,10,x) << " "
       << ib.ordered_interval(0.0,10,x) << " "
       << ib.bsearch_inc(0.0,x,0,10) << endl; 
  cout << ib.find_inc(0.9999,10,x) << " "
       << ib.ordered_lookup(0.9999,10,x) << " "
       << ib.ordered_interval(0.9999,10,x) << " "
       << ib.bsearch_inc(0.9999,x,0,10) << endl; 
  cout << ib.find_inc(1.0,10,x) << " "
       << ib.ordered_lookup(1.0,10,x) << " "
       << ib.ordered_interval(1.0,10,x) << " "
       << ib.bsearch_inc(1.0,x,0,10) << endl; 
  cout << ib.find_inc(1.5,10,x) << " "
       << ib.ordered_lookup(1.5,10,x) << " "
       << ib.ordered_interval(1.5,10,x) << " "
       << ib.bsearch_inc(1.5,x,0,10) << endl; 
  cout << ib.find_inc(5.0,10,x) << " "
       << ib.ordered_lookup(5.0,10,x) << " "
       << ib.ordered_interval(5.0,10,x) << " "
       << ib.bsearch_inc(5.0,x,0,10) << endl; 
  cout << ib.find_inc(5.5,10,x) << " "
       << ib.ordered_lookup(5.5,10,x) << " "
       << ib.ordered_interval(5.5,10,x) << " "
       << ib.bsearch_inc(5.5,x,0,10) << endl; 
  cout << ib.find_inc(8.0,10,x) << " "
       << ib.ordered_lookup(8.0,10,x) << " "
       << ib.ordered_interval(8.0,10,x) << " "
       << ib.bsearch_inc(8.0,x,0,10) << endl; 
  cout << ib.find_inc(8.5,10,x) << " "
       << ib.ordered_lookup(8.5,10,x) << " "
       << ib.ordered_interval(8.5,10,x) << " "
       << ib.bsearch_inc(8.5,x,0,10) << endl; 
  cout << ib.find_inc(9.0,10,x) << " "
       << ib.ordered_lookup(9.0,10,x) << " "
       << ib.ordered_interval(9.0,10,x) << " "
       << ib.bsearch_inc(9.0,x,0,10) << endl; 
  cout << ib.find_inc(9.5,10,x) << " "
       << ib.ordered_lookup(9.5,10,x) << " "
       << ib.ordered_interval(9.5,10,x) << " "
       << ib.bsearch_inc(9.5,x,0,10) << endl; 
  cout << endl;

  for(i=0;i<10;i++) {
    x[i]=9.0-((double)(i));
  }
  
  cout << ib.find_dec(-1.0,10,x) << " " 
       << ib.ordered_lookup(-1.0,10,x) << " " 
       << ib.ordered_interval(-1.0,10,x) << " " 
       << ib.bsearch_dec(-1.0,x,0,10) << endl; 
  cout << ib.find_dec(0.0,10,x) << " "
       << ib.ordered_lookup(0.0,10,x) << " "
       << ib.ordered_interval(0.0,10,x) << " "
       << ib.bsearch_dec(0.0,x,0,10) << endl; 
  cout << ib.find_dec(0.9999,10,x) << " "
       << ib.ordered_lookup(0.9999,10,x) << " "
       << ib.ordered_interval(0.9999,10,x) << " "
       << ib.bsearch_dec(0.9999,x,0,10) << endl; 
  cout << ib.find_dec(1.0,10,x) << " " 
       << ib.ordered_lookup(1.0,10,x) << " " 
       << ib.ordered_interval(1.0,10,x) << " " 
       << ib.bsearch_dec(1.0,x,0,10) << endl; 
  cout << ib.find_dec(1.5,10,x) << " " 
       << ib.ordered_lookup(1.5,10,x) << " " 
       << ib.ordered_interval(1.5,10,x) << " " 
       << ib.bsearch_dec(1.5,x,0,10) << endl; 
  cout << ib.find_dec(5.0,10,x) << " " 
       << ib.ordered_lookup(5.0,10,x) << " " 
       << ib.ordered_interval(5.0,10,x) << " " 
       << ib.bsearch_dec(5.0,x,0,10) << endl; 
  cout << ib.find_dec(5.5,10,x) << " " 
       << ib.ordered_lookup(5.5,10,x) << " " 
       << ib.ordered_interval(5.5,10,x) << " " 
       << ib.bsearch_dec(5.5,x,0,10) << endl; 
  cout << ib.find_dec(8.0,10,x) << " " 
       << ib.ordered_lookup(8.0,10,x) << " " 
       << ib.ordered_interval(8.0,10,x) << " " 
       << ib.bsearch_dec(8.0,x,0,10) << endl; 
  cout << ib.find_dec(8.5,10,x) << " " 
       << ib.ordered_lookup(8.5,10,x) << " " 
       << ib.ordered_interval(8.5,10,x) << " " 
       << ib.bsearch_dec(8.5,x,0,10) << endl; 
  cout << ib.find_dec(9.0,10,x) << " " 
       << ib.ordered_lookup(9.0,10,x) << " " 
       << ib.ordered_interval(9.0,10,x) << " " 
       << ib.bsearch_dec(9.0,x,0,10) << endl; 
  cout << ib.find_dec(9.5,10,x) << " " 
       << ib.ordered_lookup(9.5,10,x) << " " 
       << ib.ordered_interval(9.5,10,x) << " " 
       << ib.bsearch_dec(9.5,x,0,10) << endl; 
  cout << endl;

  t.report();
  return 0;
}

