/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/cold_nstar.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;

#include <o2scl/test_mgr.h>
#include <o2scl/rmf_eos.h>
#include <o2scl/skyrme_eos.h>
#include <o2scl/gen_potential_eos.h>
#include <o2scl/nonrel_fermion.h>
#include <o2scl/gsl_mroot_hybrids.h>
#include <o2scl/base_ioc.h>
#include <o2scl/schematic_eos.h>

int main(void) {

  cout.setf(ios::scientific);

  cold_nstar nst;
  skyrme_eos sk;
  nonrel_fermion n(o2scl_fm::mass_neutron,2.0);
  nonrel_fermion p(o2scl_fm::mass_proton,2.0);
  n.non_interacting=false;
  p.non_interacting=false;
  nst.set_n_and_p(n,p);

  test_mgr t;
  t.set_output_level(1);

  base_ioc bio;
  table_io_type table_io;

  collection co;

  gen_potential_eos go;
  
  go.B=106.35/hc_mev_fm;
  go.sigma=4.0/3.0;
  go.rho0=0.16;
  go.Cu=-103.40/hc_mev_fm;
  go.Cl=-11.70/hc_mev_fm;
  go.form=go.mdi_form;
  go.Lambda=cbrt(1.5*pi2*go.rho0);
  
  go.x=0.0;
  
  go.Au=-95.98/hc_mev_fm-2.0*go.B*go.x/(go.sigma+1.0);
  go.Al=-120.75/hc_mev_fm+2.0*go.B*go.x/(go.sigma+1.0);
  
  nst.set_eos(go);
  nst.calc_eos();
  
  nst.calc_nstar();
  
  table &tr=nst.get_tov_results();
  cout << " M_{max} = " << tr.max("gm") << " R_{max} = "
       << tr.get("r",tr.lookup("gm",tr.max("gm"))) << " cent. density = "
       << tr.get("nb",tr.lookup("gm",tr.max("gm"))) << endl;
  
  text_out_file *tof=new text_out_file("cold_nstar.1.dat");
  co.out_one(tof,"table","tov",&tr,0,0);
  delete tof;
  
  t.report();
  return 0;
}

