/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifndef O2SCL_EOS_H
#define O2SCL_EOS_H

#include <o2scl/part.h>
#include <o2scl/deriv.h>
#include <o2scl/mroot.h>
#include <o2scl/collection.h>

#ifndef DOXYGENP
namespace o2scl {
#endif

  /** 
      \brief Equation of state base
    
      A base class for the computation of an equation of state

  */
  class eos {
  public:

    eos();
    virtual ~eos() {};

    /// Set class thermo object
    virtual int set_thermo(thermo &th);

    /// Get class thermo object
    virtual int get_thermo(thermo *&th);

    /// The default thermo object
    thermo def_thermo;

    /// Return string denoting type ("eos")
    virtual const char *type() { return "eos"; }

  protected:

#ifndef DOXYGEN_INTERNAL

    friend class io_tlate<eos>;
    
    /// A pointer to the thermo object
    thermo *eos_thermo;

#endif

  };
  
  template<> int io_tlate<eos>::input
    (cinput *co, in_file_format *ins, eos *ep);
  template<> int io_tlate<eos>::output
    (coutput *co, out_file_format *outs, eos *ep);
  template<> const char *io_tlate<eos>::type();
  
  typedef io_tlate<eos> eos_io_type;

#ifndef DOXYGENP
}
#endif

#endif
