/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/funct.h>
#include <o2scl/gsl_inte_qags.h>

using namespace std;
using namespace o2scl;

double testfun3(double x, double &alpha) {
  double f=log(alpha*x)/sqrt(x);
  return f;
  
}

double f(double x, void *params) {
  double alpha=*(double *)params;
  double f=log(alpha*x)/sqrt(x);
  return f;
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  gsl_inte_qags<double,funct_fptr_noerr<double> > it6;

  double ans, exact;

  cout.setf(ios::scientific);
  cout.precision(10);

  funct_fptr_noerr<double> tf3(testfun3);

  cout << "gsl_inte_qags:" << endl;
  double alf=1.0, herr;
  exact=-4.0;
  ans=it6.integ(tf3,0.0,1.0,alf);
  herr=it6.get_error();
  t.test_rel(ans,exact,1.0e-8,"qags test");
  
  // Compare with GSL and make sure we get the same result
  {
    gsl_integration_workspace *w=gsl_integration_workspace_alloc(1000);
    
    double result, error;
    double alpha = 1.0;
    
    gsl_function F;
    F.function = &f;
    F.params = &alpha;
    
    gsl_integration_qags(&F,0,1,0,1e-7,1000,w,&result,&error); 
    
    t.test_rel(ans,result,1.0e-11,"GSL vs. O2scl");
    cout << ans << " " << herr << endl;
    cout << result << " " << error << endl;
    std::string s1=double_to_ieee_string(&ans);
    std::string s2=double_to_ieee_string(&result);
    std::string s3=double_to_ieee_string(&herr);
    std::string s4=double_to_ieee_string(&error);
    cout << s1 << "\n" << s2 << endl;
    cout << s3 << "\n" << s4 << endl;
    
    gsl_integration_workspace_free(w);
  }

  
  t.report();
  return 0;
}
