/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/test_mgr.h>
#include <o2scl/multi_funct.h>
#include <o2scl/gsl_monte.h>

using namespace std;
using namespace o2scl;

double test_fun(size_t nv, const ovector_view &x, void *&vp) {
  return x[0]*x[0]*x[1];
}

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  void *vpx=0;
 
  cout.setf(ios::scientific);

  gsl_monte<void *,multi_funct<void *> > gm;
  ovector a(2), b(2);
  a[0]=0.0;
  a[1]=0.0;
  b[0]=2.0;
  b[1]=2.0;

  multi_funct_fptr_noerr<void *> tf(test_fun);

  double res, err;
  gm.minteg_err(tf,2,a,b,vpx,res,err);
  cout << res << " " << 16.0/3.0 << endl;
  cout << err << endl;
  t.test_rel(res,16.0/3.0,1.0e-2,"gsl_monte");
  t.report();
 
  return 0;
}


