/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#include <o2scl/nuclear_mass.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;
using namespace o2scl_fm;

#include <iostream>
#include <o2scl/test_mgr.h>

using namespace o2scl_fm;

int main(void) {

  test_mgr t;
  t.set_output_level(2);

  cout.setf(ios::scientific);
  
  ame_mass au;
  semi_empirical_mass sm;
  mnms95_mass m95;

  {
    int tN, tZ, tA; 
    au.parse_elstring("Uuo300",tZ,tN,tA);
    t.test_gen(tZ==118,"parse1");
    t.test_gen(tA==300,"parse2");
    au.parse_elstring("Pb208",tZ,tN,tA);
    t.test_gen(tZ==82,"parse1");
    t.test_gen(tA==208,"parse2");
    au.parse_elstring("H4",tZ,tN,tA);
    t.test_gen(tZ==1,"parse1");
    t.test_gen(tA==4,"parse2");
  }

  t.test_rel(au.binding_energy(82,126)/208.0,-7.867,1.0e-4,"au be");
  t.test_rel(sm.binding_energy(82,126)/208.0,-7.867,5.0e-2,"sm be");
  t.test_rel(m95.binding_energy(82,126)/208.0,-7.867,5.0e-4,"m95 be");

  cout << au.mass_excess(82,126) << endl;
  cout << au.binding_energy(82,126) << endl;
  cout << au.total_mass(82,126) << endl;
  cout << sm.mass_excess(82,126) << endl;
  cout << sm.binding_energy(82,126) << endl;
  cout << sm.total_mass(82,126) << endl;
  
  nucleus n;
  au.get_nucleus(82,126,n);
  t.test_rel(n.be*o2scl_const::hc_mev_fm/208.0,-7.867,1.0e-4,"au be2");
  t.test_rel(n.m-126.0*mass_neutron-82.0*mass_proton,n.be,1.0e-12,"au be3");
  t.test_rel(n.m+82.0*mass_electron-208.0*mass_amu,n.mex,1.0e-12,
	     "au mex");

  mnms95_mass mm;
  mnms95_mass_entry mme;

  mme=mm.get_ZN(82,126);
  cout << mme.Mth << " " << mme.Mexp << endl;
  
  ame_mass am;
  ame_entry ae;
  ae=am.get_ZN(82,126);
  cout << ae.mass/1.0e3 << endl;

  cout << endl;
  for(int bZ=82;bZ<85;bZ++) {
    for(int bN=125;bN<=127;bN++) {
      ame_entry betest;

      betest=am.get_ZN(bZ,bN);
      cout << betest.bde/1.0e3 << " ";
      double bmass1=betest.mass/1.0e3+(bZ+bN)*mass_amu*hc_mev_fm;
      
      betest=am.get_ZN(bZ+1,bN-1);
      double bmass2=betest.mass/1.0e3+(bZ+bN)*mass_amu*hc_mev_fm;
      
      cout << bmass1-bmass2 << endl;
    }
  }
  
  ame_entry lead;

  {
    ame_mass ax1;
    lead=ax1.get_ZN(82,126);
    cout << lead.amass << " " << lead.be << " " << lead.beoa << endl;
    t.test_rel(lead.beoa,7867.5,2.0e-4,"lead be 1");
    ame_mass ax2("03round");
    lead=ax2.get_ZN(82,126);
    cout << lead.amass << " " << lead.be << " " << lead.beoa << endl;
    t.test_rel(lead.beoa,7867.5,2.0e-4,"lead be 2");
    ame_mass ax3("95rmd");
    lead=ax3.get_ZN(82,126);
    cout << lead.amass << " " << lead.be << " " << lead.beoa << endl;
    t.test_rel(lead.beoa,7867.5,2.0e-4,"lead be 3");
    ame_mass ax4("95exp");
    lead=ax4.get_ZN(82,126);
    cout << lead.amass << " " << lead.be << " " << lead.beoa << endl;
    t.test_rel(lead.beoa,7867.5,2.0e-4,"lead be 4");
  }
  
  ame_mass a("95rmd");
  cout << a.reference << endl;
  cout << a.n << endl;
  cout << a.is_loaded() << endl;
  t.test_gen(a.n==2931,"ame.n");

  int i;
  for(i=0;i<a.n;i+=100) {
    cout.setf(ios::right);
    cout.width(4);
    cout << i << ". ";
    cout.width(2);
    cout.setf(ios::left);
    cout << a.mass[i].el.c_str() << " ";
    cout.width(6);
    cout << a.mass[i].orig.c_str() << " "; 
    cout.width(9);
    cout << a.mass[i].amass << " ";
    cout.width(9);
    cout << a.mass[i].damass << " ";
    cout.width(9);
    cout << a.mass[i].be << " ";
    cout.width(9);
    cout << a.mass[i].beoa << endl;
  }

  i=a.n-1;

  cout.setf(ios::right);
  cout.width(4);
  cout << i << ". ";
  cout.width(2);
  cout.setf(ios::left);
  cout << a.mass[i].el.c_str() << " ";
  cout.width(6);
  cout << a.mass[i].orig.c_str() << " "; 
  cout.width(9);
  cout << a.mass[i].amass << " ";
  cout.width(9);
  cout << a.mass[i].damass << endl;
  
  cout << endl;

  lead=a.get_ZN(82,126);

  cout.setf(ios::right);
  cout.width(2);
  cout.setf(ios::left);
  cout << lead.el.c_str() << " ";
  t.test_str(lead.el,"Pb","Lead abbreviation");
  cout.width(6);
  cout << lead.orig.c_str() << " "; 
  cout.width(11);
  cout << lead.amass << " ";
  t.test_rel(lead.amass,9.766e5,1.0e-2,"Lead atomic mass");
  cout.width(11);
  cout << lead.damass << endl;
  
  lead=a.get_ZA(82,208);

  cout.setf(ios::right);
  cout.width(2);
  cout.setf(ios::left);
  cout << lead.el.c_str() << " ";
  t.test_str(lead.el,"Pb","Lead abbreviation");
  cout.width(6);
  cout << lead.orig.c_str() << " "; 
  cout.width(11);
  cout << lead.amass << " ";
  t.test_rel(lead.amass,9.766e5,1.0e-2,"Lead atomic mass");
  cout.width(11);
  cout << lead.damass << endl;
   
  lead=a.get_elA("Pb",208);

  cout.setf(ios::right);
  cout.width(2);
  cout.setf(ios::left);
  cout << lead.el.c_str() << " ";
  t.test_str(lead.el,"Pb","Lead abbreviation");
  cout.width(6);
  cout << lead.orig.c_str() << " "; 
  cout.width(11);
  cout << lead.amass << " ";
  t.test_rel(lead.amass,9.766e5,1.0e-2,"Lead atomic mass");
  cout.width(11);
  cout << lead.damass << endl;
 
  lead=a.get("Pb208");

  cout.setf(ios::right);
  cout.width(2);
  cout.setf(ios::left);
  cout << lead.el.c_str() << " ";
  t.test_str(lead.el,"Pb","Lead abbreviation");
  cout.width(6);
  cout << lead.orig.c_str() << " "; 
  cout.width(11);
  cout << lead.amass << " ";
  t.test_rel(lead.amass,9.766e5,1.0e-2,"Lead atomic mass");
  cout.width(11);
  cout << lead.damass << endl;
 
  ame_mass a2("95exp");
  cout << a2.reference << endl;
  cout << a2.n << endl;
  cout << a2.is_loaded() << endl;

  for(i=0;i<a2.n;i+=100) {
    cout.setf(ios::right);
    cout.width(4);
    cout << i << ". ";
    cout.width(2);
    cout.setf(ios::left);
    cout << a2.mass[i].el.c_str() << " ";
    cout.width(6);
    cout << a2.mass[i].orig.c_str() << " "; 
    cout.width(9);
    cout << a2.mass[i].amass << " ";
    cout.width(9);
    cout << a2.mass[i].damass << " ";
    cout.width(9);
    cout << a2.mass[i].be << " ";
    cout.width(9);
    cout << a2.mass[i].beoa << endl;
  }
  
  ame_mass a3("03round");
  cout << a3.reference << endl;
  cout << a3.n << endl;
  cout << a3.is_loaded() << endl;

  for(i=0;i<a3.n;i+=100) {
    cout.setf(ios::right);
    cout.width(4);
    cout << i << ". ";
    cout.width(2);
    cout.setf(ios::left);
    cout << a3.mass[i].el.c_str() << " ";
    cout.width(6);
    cout << a3.mass[i].orig.c_str() << " "; 
    cout.width(9);
    cout << a3.mass[i].amass << " ";
    cout.width(9);
    cout << a3.mass[i].damass << " ";
    cout.width(9);
    cout << a3.mass[i].be << " ";
    cout.width(9);
    cout << a3.mass[i].beoa << endl;
  }

  ame_mass a4("03");
  cout << a4.reference << endl;
  cout << a4.n << endl;
  cout << a4.is_loaded() << endl;

  for(i=0;i<a4.n;i+=100) {
    cout.setf(ios::right);
    cout.width(4);
    cout << i << ". ";
    cout.width(2);
    cout.setf(ios::left);
    cout << a4.mass[i].el.c_str() << " ";
    cout.width(6);
    cout << a4.mass[i].orig.c_str() << " "; 
    cout.width(9);
    cout << a4.mass[i].amass << " ";
    cout.width(9);
    cout << a4.mass[i].damass << " ";
    cout.width(9);
    cout << a4.mass[i].be << " ";
    cout.width(9);
    cout << a4.mass[i].beoa << endl;
  }

  if (true) {

    void *vp;
    int sz, sz2;
    
    string file;
    collection co;
    
    base_ioc bio;
    mnms95_mass_entry_io_type mio;
    file="../../data/o2scl/nndc/moller.dat";
    
    text_in_file *tif=new text_in_file(file.c_str());
    
    co.fin(tif,0,1);
    
    string mm_name;
    collection::type_iterator it=co.begin("mnms95_mass_entry");
    mm_name=it.name();
    sz=it->size;
    sz2=it->size2;
    vp=it->data;
    
    mnms95_mass_entry *mass=(mnms95_mass_entry *)vp;
    
    co.disown(mm_name);
    co.clear();

    delete tif;

  }

  t.set_output_level(1);
  for(int Z=20;Z<220;Z+=20) {
    for(int N=20;N<220;N+=20) {
      mme=mm.get_ZN(Z,N);
      bool b1=mm.is_included(Z,N);
      bool b2=(mme.Z>0);
      t.test_gen(b1==b2,"test is_included and get_ZN");
    }
  }

  {
    // Temporary stuff
    cout << mm.is_included(8,2) << endl;
    
    ame_mass am1("95exp");
    ame_mass am2("95rmd");
    mnms95_mass mnm;
    cout << am1.is_loaded() << " " << am2.is_loaded() << " " << mm.is_loaded()
	 << endl;
    for(int Z=20;Z<220;Z+=20) {
      for(int N=20;N<220;N+=20) {
	if (am1.is_included(Z,N) && am2.is_included(Z,N)) {
	  mnms95_mass_entry mne=mnm.get_ZN(Z,N);
	  cout << Z << " ";
	  cout.width(3);
	  cout << N << " "
	       << am1.mass_excess(Z,N) << " "
	       << am2.mass_excess(Z,N) << " "
	       << mnm.mass_excess(Z,N) << " " << mne.Mth << " ";
	  cout.setf(ios::showpos);
	  cout << mne.Mexp << endl;
	  cout.unsetf(ios::showpos);
	}
      }
    }

    for(int Z=5;Z<17;Z++) {
      int N=21-Z;
      cout << Z << " " << N << " " << am1.is_included(Z,N) << " "
	   << am1.mass_excess(Z,N) << endl;
    }
      
  }

  t.report();
  return 0;
}

