/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/

#include <o2scl/other_ioc.h>
#include <o2scl/gsl_series.h>

/// For the definition of M_PI
#include <gsl/gsl_math.h>

using namespace std;
using namespace o2scl;

int main(void) {
  test_mgr t;
  t.set_output_level(2);
  const int N=20;
  double arr[N], sum, err;
  gsl_series gs(N);

  cout.setf(ios::scientific);
  cout.precision(14);

  const double zeta_2=M_PI*M_PI/6.0;
  for (int n=0;n<N;n++) {
    double np1=n+1.0;
    arr[n]=1.0/(np1*np1);
  }
  
  sum=gs.series_accel(arr,err);
  t.test_rel(sum,zeta_2,err,"lewin");
  cout << sum << " " << " " << err << " " << zeta_2 << endl;

  cout << "Test changing the size:" << endl;
  gs.set_size(N-10);
  sum=gs.series_accel(arr,err);
  t.test_rel(sum,zeta_2,err,"lewin");
  cout << sum << " " << " " << err << " " << zeta_2 << endl;

  other_ioc oio;

  void *vp;
  gsl_series *gs2;
  collection co;
  co.test_type(t,"gsl_series",&gs,vp,1);
  gs2=(gsl_series *)vp;

  t.report();
  return 0;
}
