/*
  -------------------------------------------------------------------
  
  Copyright (C) 2006, 2007, Andrew W. Steiner
  
  This file is part of O2scl.
  
  O2scl is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 3 of the License, or
  (at your option) any later version.
  
  O2scl is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with O2scl. If not, see <http://www.gnu.org/licenses/>.

  -------------------------------------------------------------------
*/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <o2scl/quark.h>

using namespace std;
using namespace o2scl;
using namespace o2scl_const;

//--------------------------------------------
// quark class

quark::quark(double mass, double dof) : part(mass,dof), fermion(mass,dof) {
  B=0.0;
  qq=0.0;
  name="quark";
}

int quark::calc_mu(const double temper) {
  set_err_ret("Missing base function quark::calc_mu().",gsl_nobase);
}

int quark::calc_density(const double temper) {
  set_err_ret("Missing base function quark::calc_density().",gsl_nobase);
}

int quark::pair_mu(const double temper) {
  set_err_ret("Missing base function quark::pair_mu().",gsl_nobase);
}

int quark::pair_density(const double temper) {
  set_err_ret("Missing base function quark::pair_density().",gsl_nobase);
}

template<> int io_tlate<quark>::input
(cinput *co, in_file_format *ins, quark *q) {
  part *p=(part *)q;
  std::string t;
  co->object_in("part",ins,p,t);
  ins->double_in(q->B,"B");
  ins->double_in(q->qq,"qq");
  return 0;
}

template<> int io_tlate<quark>::output
(coutput *co, out_file_format *outs, quark *q) {
  part *p=(part *)q;
  std::string t;
  co->object_out("part",outs,p,0,0,t);
  outs->double_out(q->B,"B");
  outs->double_out(q->qq,"qq");
  return 0;
}

template<> const char *io_tlate<quark>::type()
{ return "quark"; }
